#ifndef __COCLSMAP_H__
#define __COCLSMAP_H__

#include <windows.h>

/**************************************************************************
NAME         : g_cLocks
TYP          : globale Funktion
AUFGABE      : Kapselung des Sperrungsstatus der DLL; verhindert das vorzeitige Entladen der DLL
RETVAL       : Zeiger auf die Anzahl der durchgefhrten Sperrungen
**************************************************************************/
LONG *g_cLocks();

/**************************************************************************
NAME         : IMPLEMENT_ICLASSFACTORY
TYP          : Makro
AUFGABE      : Implementiert die Schnittstelle IClassFactory (fr statische COM-Klassen)
IN           : InstCls ... Name der Instanzklasse
**************************************************************************/
#define IMPLEMENT_ICLASSFACTORY(InstCls) \
	STDMETHODIMP CreateInstance(IUnknown *pUnkOuter, REFIID riid, void **ppvObject) \
	{ \
		if (pUnkOuter) return CLASS_E_NOAGGREGATION; \
		InstCls *instance=new InstCls(); \
		if (instance) \
		{ \
			instance->AddRef(); \
			HRESULT hr=instance->QueryInterface(riid,ppvObject); \
			instance->Release(); \
			return hr; \
		} \
		return E_OUTOFMEMORY; \
	} \
	\
	STDMETHODIMP LockServer(BOOL fLock) \
	{ \
		if (fLock) \
		{ \
			InterlockedIncrement(g_cLocks()); \
		} else { \
			if (*g_cLocks()>0) \
			{ \
				InterlockedDecrement(g_cLocks()); \
			} else { \
				return E_FAIL; \
			} \
		} \
		return S_OK; \
	} \

/**************************************************************************
NAME         : IMPLEMENT_CANUNLOADNOW
TYP          : Makro
AUFGABE      : Implementiert DllCanUnloadNow mittels der globalen Funktion g_cLocks
**************************************************************************/
#define IMPLEMENT_CANUNLOADNOW() \
	LONG *g_cLocks() \
	{ \
		typedef struct tagLONG0 { \
			LONG Value; \
			tagLONG0() { Value=0; } \
		} LONG0; \
		static LONG0 cLocks; \
		return &cLocks.Value; \
	} \
	\
	STDAPI DllCanUnloadNow() \
	{ \
		return (*g_cLocks()==0)?S_OK:S_FALSE; \
	} \

/**************************************************************************
NAME         : BEGIN_IMPLEMENT_GETCLASSOBJECT
TYP          : Makro (Methodenkopf)
AUFGABE      : Definiert den Funktionskopf von DllGetClassObject
**************************************************************************/
#define BEGIN_IMPLEMENT_GETCLASSOBJECT() \
	STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv) \
	{ \

/**************************************************************************
NAME         : MAP_COCLASS
TYP          : Makro (Methodenrumpf)
AUFGABE      : Integration von statischen COM-Klassen und ihrer CLSIDs in DllGetClassObject
IN           : CoCls ... Name der einzutragenden COM-Klasse
**************************************************************************/
#define MAP_COCLASS(CoCls) \
		static const CoCls _##CoCls_; \
		if (IsEqualGUID(rclsid,CLSID_##CoCls)) return (const_cast<IClassFactory*>((const IClassFactory*)&_##CoCls_))->QueryInterface(riid, ppv); \

/**************************************************************************
NAME         : MAP_COCLASS_AS
TYP          : Makro (Methodenrumpf)
AUFGABE      : Integration von statischen COM-Klassen (mit Aliasnamen) und ihrer CLSIDs in DllGetClassObject
IN           : CoCls ... Klassename der einzutragenden Klasse
               alias ... COM-Name der einzutragenden Klasse
**************************************************************************/
#define MAP_COCLASS_AS(alias, CoCls) \
		static const CoCls _##CoCls_; \
		if (IsEqualGUID(rclsid,CLSID_##alias)) return (const_cast<IClassFactory*>((const IClassFactory*)&_##CoCls_))->QueryInterface(riid, ppv); \

/**************************************************************************
NAME         : END_IMPLEMENT_GETCLASSOBJECT
TYP          : Makro (Methodenende)
AUFGABE      : Definiert den Funktionsabschluss von DllGetClassObject
**************************************************************************/
#define END_IMPLEMENT_GETCLASSOBJECT() \
		*ppv=NULL; \
		return CLASS_E_CLASSNOTAVAILABLE; \
	} \

#endif